#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "iio.h"
#include "Imagenes.h"

//#define PRINT_HISTO
#define ABS(x) (x<0 ? (-x):(x))

int main(int argc, char** argv){
  int error = 1;
/*  printf("argc %d \n",argc);*/
  if(argc >= 3 && argc <= 4){
    if((strcmp(argv[2],"RG")==0) ||
       (strcmp(argv[2],"RSP")==0) ||
       (strcmp(argv[2],"P")==0) ||
       (strcmp(argv[2],"M")==0)){
      if (argc == 4)
	error = 0;
      else
	error = 1;
    }
    if((strcmp(argv[2],"G")==0) ||
       (strcmp(argv[2],"H")==0) ||
       (strcmp(argv[2],"E")==0) ||
       (strcmp(argv[2],"A")==0)){
      if (argc == 3)
	error = 0;
      else
	error = 1;
    }
  }

  if(error){
        printf("Uso incorrecto del ejecutable. Se debe ingresar una entrada \n"
	   "con la siguiente forma:\n "
	   "./Test archivoEntrada opcion valor \n"
	   "donde:\n"
	       "archivoSalida: contiene la ruta del archivo de entrada.\n"
	       "opcion: \n"
	       "G: convertir a gris\n"
	       "E: ecualizar\n"
	       "RG: ruido gaussiano [potencia]\n"
	       "RSP: ruido sal y pimienta [potencia]\n"
	       "H: histograma\n"
	       "A: histograma acumulado\n"
	       "P: filtro promediador [orden]\n"
	       "M: filtro mediana [orden]\n"
	       );
    return -1;
  }
  Imagen img, imgGris, imgOut;

  if(leerImagen(argv[1], &img) != OK){
    printf("error leer\n");
    return(-1);
  }

  if(strcmp(argv[2],"G")==0){
    if(img.tipo == COLOR){
      printf("Convierto a gris\n");
      if(convertirImagenColorAGris(&img,&imgGris) != OK){
	printf("error convierto a gris\n");
	return(-1);
      }
      if(escribirImagen(&imgGris,"ConvertidaAGris.png") != OK){
	printf("error escribir\n");
	return(-1);
      }
    }
    else{
      if(escribirImagen(&img,"ImagenGris.png") != OK){
	printf("error escribir\n");
	return(-1);
      }
    }
  }

  if(strcmp(argv[2],"G") != 0){
    printf("Siempre Convierto a gris\n");
    if(convertirImagenColorAGris(&img,&imgGris) != OK){
      printf("error convierto a gris\n");
      return(-1);
    }
  }

  if(strcmp(argv[2],"H")==0){
    printf("Histograma\n");
    long int *histo = NULL;
    if((histo = Histo(&imgGris)) == NULL)
      printf("error al generar histograma\n");

#ifdef DEBUG
    for(int i=0;i<255;i++)
      printf("%d %ld\n",i,histo[i]);
#endif
    Plot_Histo(histo, 10, 0.01);
    free(histo);
  }

  if(strcmp(argv[2],"A")==0){
    printf("Histograma acumulado\n");
    
    int f = imgGris.filas;
    int c = imgGris.columnas;
    int size = f*c;
  
    long int *histoAcum; // = (long int*)malloc(sizeof(long int)*256);
    long int *histo = NULL;
    if((histo = Histo(&imgGris)) == NULL)
      printf("error al generar histograma\n");
    printf("histograma:\n");
    Plot_Histo(histo, 10, 0.01);
    if((histoAcum = HistoAcum(histo, size)) == NULL){
      printf("error en HistoAcum\n");
      exit(-1);
    }
    printf("histograma acumulado:\n");
    Plot_Histo(histoAcum, 10, 0.0001);

#ifdef PRINT_HISTO
    if( imprimirHisto(histo) != OK){
      printf("error en imprimir histograma\n");
      exit(-1);
    }
#endif

  }

  if(strcmp(argv[2],"E")==0){
    printf("Ecualizo\n");
    
    int f = imgGris.filas;
    int c = imgGris.columnas;
    int size = f*c;
  
    long int *histoAcum; 
    long int *histo = NULL;
    if((histo = Histo(&imgGris)) == NULL)
      printf("error al generar histograma\n");
    printf("histograma:\n");
    Plot_Histo(histo, 10, 0.01);
    if((histoAcum = HistoAcum(histo, size)) == NULL){
      printf("error en HistoAcum\n");
      exit(-1);
    }
    printf("histograma acumulado:\n");
    Plot_Histo(histoAcum, 10, 0.01);

#ifdef PRINT_HISTO
    if( imprimirHisto(histo) != OK){
      printf("error en imprimir histograma\n");
      exit(-1);
    }
#endif
    if (LUT(&imgGris, &imgOut, histoAcum) != OK){
      printf("error en LUT\n");
      exit(-1);
    }
    
    if(escribirImagen(&imgOut,"Ecualizada.png") != OK){
      printf("error escribir\n");
      return(-1);
    }
  }

  if(strcmp(argv[2],"RG")==0){
    printf("Ruido Gaussiano potencia %f\n",atof(argv[3]));
    if(AgregarRudioGaussiano(&imgGris, &imgOut, atof(argv[3]), 12345) != OK){
      printf("error ruido sal y pimienta\n");
      return(-1);
    }
    
    if(escribirImagen(&imgOut,"ConGaussiano.png") != OK){
      printf("error escribir\n");
      return(-1);
    }
  }

  if(strcmp(argv[2],"RSP")==0){
    printf("Ruido Sal y Pimienta potencia %f\n",atof(argv[3]));
    if(AgregarRudioSalYPimienta(&imgGris, &imgOut, atof(argv[3]), 12345) != OK){
      printf("error ruido sal y pimienta\n");
      return(-1);
    }
    
    if(escribirImagen(&imgOut,"ConRSP.png") != OK){
      printf("error escribir\n");
      return(-1);
    }
  }

  if(strcmp(argv[2],"P")==0){
    printf("Filtro promediador orden %d\n",atoi(argv[3]));
    if( FiltroPromedio(&imgGris,&imgOut, atoi(argv[3])) != OK){
      printf("error ruido sal y pimienta\n");
      return(-1);
    }
    
    if(escribirImagen(&imgOut,"Promedio.png") != OK){
      printf("error escribir\n");
      return(-1);
    }
  }

  if(strcmp(argv[2],"M")==0){
    printf("Filtro mediana orden %d\n",atoi(argv[3]));

    if( FiltroMediana(&imgGris,&imgOut, atoi(argv[3])) != OK){
      printf("error ruido sal y pimienta\n");
      return(-1);
    }
    
    if(escribirImagen(&imgOut,"Mediana.png") != OK){
      printf("error escribir\n");
      return(-1);
    }
  }

  if(destruirImagen(&img) != OK){
    printf("error destruir\n");
    return(-1);
  }
  if(destruirImagen(&imgOut) != OK){
    printf("error destruir\n");
    return(-1);
  }
  
  if(destruirImagen(&imgGris) != OK){
    printf("error destruir\n");
    return(-1);
  }

  return(0);
}

